@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -XScopedTypeVariables #-}
+{-# LANGUAGE ScopedTypeVariables, PatternGuards #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Metainfo.WorkspaceCollector
@@ -32,7 +32,9 @@ module IDE.Metainfo.WorkspaceCollector (
 import IDE.Utils.Utils
 import IDE.Utils.GHCUtils
 import GHC hiding(Id,Failed,Succeeded,ModuleName)
+#if !MIN_VERSION_ghc(7,2,0)
 import HscTypes hiding (liftIO)
+#endif
 import Outputable hiding(trace)
 import ErrUtils
 import qualified Data.Map as Map
@@ -41,7 +43,6 @@ import System.Directory
 import Distribution.Package hiding (PackageId)
 import Distribution.ModuleName
 import Distribution.Text (simpleParse)
-import Control.Monad.Reader
 import System.FilePath
 import qualified Data.ByteString.Char8 as BS
 import Data.Binary.Shared
@@ -62,13 +63,15 @@ import TcRnMonad (initTcRnIf, IfGblEnv(..))
 import qualified Maybes as M
 import IDE.Metainfo.InterfaceCollector
 import Data.Maybe
-       (isJust, fromJust, catMaybes, mapMaybe, isNothing)
+       (isJust, catMaybes, mapMaybe, isNothing)
 import Module (stringToPackageId)
 import PrelNames
 import System.Log.Logger
 import Control.DeepSeq (deepseq)
 #if MIN_VERSION_ghc(6,12,1)
 import FastString(mkFastString,appendFS,nullFS,unpackFS)
+import Control.Monad.IO.Class (MonadIO, MonadIO(..))
+import Control.Monad (when)
 #else
 import GHC.Show(showSpace)
 #endif
@@ -145,7 +148,11 @@ collectModule' :: FilePath -> FilePath -> Bool -> PackageIdentifier -> [String]
 collectModule' sourcePath destPath writeAscii packId opts moduleName' = gcatch (
     inGhcIO (opts++["-cpp"]) [Opt_Haddock] $ \ _dynFlags -> do
         session         <-  getSession
+#if MIN_VERSION_ghc(7,2,0)
+        (dynFlags3,fp') <-  liftIO $ preprocess session (sourcePath,Nothing)
+#else
         (dynFlags3,fp') <-  preprocess session (sourcePath,Nothing)
+#endif
         mbInterfaceDescr <- mayGetInterfaceDescription packId moduleName'
         liftIO $ do
             stringBuffer    <-  hGetStringBuffer fp'
@@ -325,13 +332,23 @@ finishedDoc d doc rest | notDocDecl d = (d, Just doc) : rest
     notDocDecl _              = True
 finishedDoc _ _ rest = rest
 
+#if MIN_VERSION_ghc(7,2,0)
+sigNameNoLoc :: Sig name -> Maybe name
+-- Used only in Haddock
+sigNameNoLoc (TypeSig   [n] _)          = Just (unLoc n)
+sigNameNoLoc (SpecSig   n _ _)        = Just (unLoc n)
+sigNameNoLoc (InlineSig n _)          = Just (unLoc n)
+sigNameNoLoc (FixSig (FixitySig n _)) = Just (unLoc n)
+sigNameNoLoc _                        = Nothing
+#endif
+
 attachSignatures :: [(NDecl, (Maybe NDoc))] -> [(NDecl,Maybe NDoc)]
     -> [(NDecl, (Maybe NDoc), [(NSig,Maybe NDoc)])]
 attachSignatures signatures = map (attachSignature signaturesMap)
     where
-    signaturesMap = Map.fromListWith (++)
-                        $ map (\ (L loc (SigD sig),c) -> (fromJust $ sigNameNoLoc sig, [(L loc sig,c)]))
-                                signatures
+    signaturesMap = Map.fromListWith (++) $ map sigMap signatures
+    sigMap (L loc (SigD sig),c) | Just name <- sigNameNoLoc sig = (name, [(L loc sig,c)])
+    sigMap _ = error "Unexpected location type"
     attachSignature :: Map RdrName  [(NSig,Maybe NDoc)] -> (NDecl, (Maybe NDoc))
         -> (NDecl, (Maybe NDoc), [(NSig,Maybe NDoc)])
     attachSignature signaturesMap'  (decl,mbDoc) =
@@ -406,7 +423,7 @@ transformToDescrs pm = concatMap transformToDescr
         derivings Nothing = []
         derivings (Just l) = map (extractDeriving pm name) l
 
-    transformToDescr ((L loc (TyClD cl@(ClassDecl _ tcdLName' _ _ tcdSigs' _ _ docs))), mbComment,_sigList) =
+    transformToDescr ((L loc (TyClD cl@(ClassDecl{tcdLName=tcdLName', tcdSigs=tcdSigs', tcdDocs=docs}))), mbComment,_sigList) =
         [Real $ RealDescr {
         dscName'        =   showRdrName (unLoc tcdLName')
     ,   dscMbTypeStr'   =   Just (BS.pack (showSDocUnqual $ppr cl{tcdMeths = emptyLHsBinds}))
@@ -487,7 +504,11 @@ extractMethods sigs docs =
     in mapMaybe extractMethod pairs
 
 extractMethod :: OutputableBndr alpha => (LHsDecl alpha, Maybe (NDoc)) -> Maybe SimpleDescr
+#if MIN_VERSION_ghc(7,2,0)
+extractMethod ((L loc (SigD ts@(TypeSig [name] _typ))), mbDoc) =
+#else
 extractMethod ((L loc (SigD ts@(TypeSig name _typ))), mbDoc) =
+#endif
     Just $ SimpleDescr
         ((showSDoc . ppr) (unLoc name))
         (Just (BS.pack (showSDocUnqual $ ppr ts)))
@@ -532,11 +553,18 @@ sigToByteString [(sig,_)] = Just (BS.pack (showSDocUnqual $ppr sig))
 sigToByteString ((sig,_):_) = Just (BS.pack (showSDocUnqual $ppr sig))
 
 srcSpanToLocation :: SrcSpan -> Maybe Location
+#if MIN_VERSION_ghc(7,2,0)
+srcSpanToLocation (RealSrcSpan span')
+    =   Just (Location (srcSpanStartLine span') (srcSpanStartCol span')
+                 (srcSpanEndLine span') (srcSpanEndCol span'))
+srcSpanToLocation _ = Nothing
+#else
 srcSpanToLocation span' | not (isGoodSrcSpan span')
     =   Nothing
 srcSpanToLocation span'
     =   Just (Location (srcSpanStartLine span') (srcSpanStartCol span')
                  (srcSpanEndLine span') (srcSpanEndCol span'))
+#endif
 
 toComment :: Maybe (NDoc) -> [NDoc] -> Maybe ByteString
 toComment (Just c) _    =  Just (BS.pack (printHsDoc c))
